// ----------------------------------------------------------------------------
// * Raima Database Manager
// *
// * Copyright (c) 2012 Raima Inc.,  All rights reserved.
// *
// * Use of this software, whether in source code format, or in executable,
// * binary object code form, is governed by the Raima LICENSE which
// * is fully described in the LICENSE.TXT file, included within this
// * distribution of files.
// * ----------------------------------------------------------------------------

#include <iostream>

// rdm.h is required in all files using the RDM Database
#include "rdm.h"

// obj12_db.h was generated from the DDL file obj12_db.ddl.
#include "obj12_db.h"

// obj12_db_api.h was generated from the DDL file obj12_db.ddl. with 
// the -cpp option
#include "obj12_db_api.h"

// Name space for the RDM C++ interface
using namespace RDM_CPP;

// EXAMPLE - obj12 for C++
// Each time you run this example:
//   1) The database is initialized (all existing contents removed)
//   2) A transaction is started
//   3) Several NA offices are added to the database
//   4) All offices in the database are displayed (including uncommitted)
//   5) The transaction is committed
//   6) All offices in the database are displayed (all now committed)
//   7) Another transaction is started
//   8) Several EMEA offices are added to the database
//   9) All offices in the database are displayed (including uncommitted)
//  10) The transaction is rolled back
//  11) All offices in the database are displayed (uncommitted EMEA offices
//      are no longer in the database)

// The DDL (Database Definition Language) is in obj12_db.ddl. The database
// schema is embedded into the application and the database will be created
// the first time the application is run.  Subsequent runs will use the now
// existing database.

// The following RDM C++ methods are illustrated:
//      Db->Open
//      Db->Initialize
//      Db->BeginUpdate
//      Db->End
//      Db->Abort
//      Db->BeginRead
//      Db->New_recordWithFieldValues
//      Db->Get_recordsByKey
//      Cursor->AfterLast
//      Cursor->GetFieldValues
//      Cursor++
// They are fully documented in http://docs.raima.com/rdm/.

// Each of these methods can throw and an exception of type rdm_exception

// The actual database is stored in a directory named 'obj12_db' in this
// project directory.

const char * na_offices[] =
{
    "Seattle",
    "Boise",
    "San Francisco",
    "Dallas"
};

const char * emea_offices[] =
{
    "Paris",
    "London",
    "Dublin",
    "Zurich",
    "Madrid"
};

class obj12
{
public:
    Db_obj12_db    db;

    void InsertData (uint16_t num_offices, const char *office_names[]) throw (rdm_exception&);
    void DisplayOffices () throw (rdm_exception&);
    void Initialize () throw (rdm_exception&);
    void Cleanup () throw (rdm_exception&);
};

void obj12::Initialize()
{
    // Open the North American database and initialize it
    try
    {
        db = db.Open("obj12_db");
        db.Initialize();
    }
    catch (rdm_exception& e)
    {
        cerr << "Error " << e.error() << " (" << e.what() << ") opening/initializing obj12_db database" << endl;
        throw (e);
        return;
    }
    return;
}

void obj12::InsertData(
    uint16_t    num_offices,
    const char *office_names[])
{
    int16_t       ii;
    struct office office_rec;
    
    
    try
    {
        // Create offices in the database
        for (ii=0; ii<num_offices; ii++)
        {
            strcpy(office_rec.name, office_names[ii]);
            db.New_office_recordWithFieldValues(&office_rec);
        }
    }
    catch (rdm_exception& e)
    {
        db.Abort();
        cerr << "Error " << e.error() << " (" << e.what() << ") adding records" << endl;
        throw (e);
        return;
    }
    return;
}
    
void obj12::DisplayOffices()
{
    struct office office_rec;
    Cursor_office office_cursor;
    recordType rtypes[]= {Cursor_office::Type (), Cursor::NoMoreTypes()};

    try
    {
        // Loop through each office record and display it
        office_cursor = db.Get_office_recordsBy_name();
        while (!office_cursor.AfterLast())
        {
            office_cursor.GetFieldValues(&office_rec);
            cout << '\t' << office_rec.name << endl;
            office_cursor++;
        }
    }
    catch (rdm_exception& e)
    {
        cerr << "Error " << e.error() << " (" << e.what() << ") reading records" << endl;
    }   
}

void obj12::Cleanup()
{
    return;
}

int32_t main(void)
{
    recordType rtypes[]= {Cursor_office::Type (), Cursor::NoMoreTypes()};
    obj12 obj;

    try
    {
        obj.Initialize();

        // Start a transaction and add NA offices
        obj.db.BeginUpdate(rtypes);
        obj.InsertData(RLEN(na_offices), na_offices);
        cout << endl << "All Offices before transaction commit." << endl;
        obj.DisplayOffices();

        // Commit the transaction
        obj.db.End();

        // Display the offices (need to get read locks)
        obj.db.BeginRead(rtypes);
        cout << endl << "All Offices after transaction commit." << endl;
        obj.DisplayOffices();
        obj.db.End(); // Free read locks

        // Start a transaction and add EMEA offices
        obj.db.BeginUpdate(rtypes);
        obj.InsertData(RLEN(emea_offices), emea_offices);
        cout << endl << "All Offices before transaction rollback." << endl;
        obj.DisplayOffices();

        // Rollback the transaction
        obj.db.Abort();

        // Display the offices (need to get read locks)
        obj.db.BeginRead(rtypes);
        cout << endl << "All Offices after transaction commit." << endl;
        obj.DisplayOffices();
        obj.db.End(); // Free read locks

        obj.Cleanup();
    }
    catch (rdm_exception& e)
    {
        cerr << "Error " << e.error() << " (" << e.what() << ")" << endl;
        return EXIT_FAILURE;
    }
    
    cout << endl << "Press <Enter> to continue . . .";
    getchar();
    return EXIT_SUCCESS;
}